@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP, ScopedTypeVariables #-}
 {- |
    Module      : Text.Highlighting.Kate.Common
    Copyright   : Copyright (C) 2008 John MacFarlane
@@ -12,10 +12,11 @@ Parsers used in all the individual syntax parsers.
 -}
 
 module Text.Highlighting.Kate.Common where
-#ifdef _PCRE_LIGHT
-import Text.Regex.PCRE.Light.Char8
-#else
 import Data.ByteString.UTF8 (fromString, toString)
+#ifdef _PCRE_LIGHT
+import Text.Regex.PCRE.Light
+import Data.ByteString (ByteString)
+#else
 import System.IO.Unsafe (unsafePerformIO)
 import Text.Regex.PCRE.ByteString
 #endif
@@ -23,6 +24,7 @@ import Text.Highlighting.Kate.Types
 import Text.ParserCombinators.Parsec hiding (State)
 import Data.Char (isDigit, toLower, isSpace)
 import Data.List (tails)
+import Text.Printf
 import Control.Monad.State
 import qualified Data.Set as Set
 
@@ -167,7 +169,7 @@ subDynamic "" = return ""
 
 -- convert octal escapes to the form pcre wants.  Note:
 -- need at least pcre 8.34 for the form \o{dddd}.
--- So we prefer \ddd when possible.
+-- So we prefer \ddd or \x{...}.
 convertOctal :: String -> String
 convertOctal [] = ""
 convertOctal ('\\':'0':x:y:z:rest)
@@ -177,15 +179,10 @@ convertOctal ('\\':x:y:z:rest)
 convertOctal ('\\':'o':'{':zs) =
   case break (=='}') zs of
        (ds, '}':rest) | all isOctalDigit ds && not (null ds) ->
-         if length ds <= 3
-            then '\\': pad3 ds ++ convertOctal rest
-            else case reads ('\\':'o':ds) of
-                   ((c,[]):_) -> c : convertOctal rest
-                   _          -> '\\':'o':'{': convertOctal zs
+            case reads ('0':'o':ds) of
+                 ((n :: Int,[]):_) -> printf "\\x{%x}" n ++ convertOctal rest
+                 _          -> error $ "Unable to read octal number: " ++ ds
        _  -> '\\':'o':'{': convertOctal zs
-    where pad3 [x]   = ['0','0',x]
-          pad3 [x,y] = ['0',x,y]
-          pad3 xs    = xs
 convertOctal (x:xs) = x : convertOctal xs
 
 isOctalDigit :: Char -> Bool
@@ -196,7 +193,7 @@ compileRegex :: Bool -> String -> Regex
 compileRegex caseSensitive regexpStr =
 #ifdef _PCRE_LIGHT
   let opts = [anchored, utf8] ++ [caseless | not caseSensitive]
-  in  compile ('.' : convertOctal regexpStr) opts
+  in  compile (fromString ('.' : convertOctal regexpStr)) opts
 #else
   let opts = compAnchored + compUTF8 +
                if caseSensitive then 0 else compCaseless
@@ -209,7 +206,10 @@ compileRegex caseSensitive regexpStr =
 
 matchRegex :: Regex -> String -> KateParser (Maybe [String])
 #ifdef _PCRE_LIGHT
-matchRegex r s = return $ match r s [exec_notempty]
+matchRegex r s = return $ toString' $ match r (fromString s) [exec_notempty]
+    where toString' :: Maybe [ByteString] -> Maybe [String]
+          toString' (Just xs) = Just $ fmap toString xs
+          toString' Nothing = Nothing
 #else
 matchRegex r s = case unsafePerformIO (regexec r (fromString s)) of
                       Right (Just (_, mat, _ , capts)) -> return $
@@ -291,7 +291,11 @@ pRangeDetect startChar endChar = try $ do
   return $ startChar : (body ++ [endChar])
 
 pLineContinue :: KateParser String
-pLineContinue = try $ char '\\' >> eof >> return "\\"
+pLineContinue = try $ do
+  char '\\'
+  eof
+  updateState $ \st -> st{ synStContinuation = True }
+  return "\\"
 
 pDetectSpaces :: KateParser [Char]
 pDetectSpaces = many1 (satisfy $ \c -> c == ' ' || c == '\t')
